from django.apps import apps as django_apps

EDC_CSF = "EDC_CSF"
EDC_CSF_SUPER = "EDC_CSF_SUPER"
EDC_CSF_VIEW = "EDC_CSF_VIEW"

codenames = []
app_config = django_apps.get_app_config("edc_csf")
for model_cls in app_config.get_models():
    if "historical" in model_cls._meta.label_lower:
        codenames.append(f"{app_config.name}.view_{model_cls._meta.model_name}")
    else:
        for prefix in ["add", "change", "view", "delete"]:
            codenames.append(f"{app_config.name}.{prefix}_{model_cls._meta.model_name}")
codenames.sort()



from django.db import models
from edc_constants.choices import NOT_APPLICABLE, POS_NEG_NA, YES_NO_NA


class BiosynexSemiQuantitativeCragMixin(models.Model):
    bios_crag = models.CharField(
        verbose_name="Biosynex Semi-quantitative CrAg performed?",
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text="Gaborone and Blantyre only",
        max_length=5,
    )

    crag_control_result = models.CharField(
        verbose_name="Control result",
        choices=POS_NEG_NA,
        default=NOT_APPLICABLE,
        help_text="Gaborone and Blantyre only",
        max_length=5,
    )

    crag_t1_result = models.CharField(
        verbose_name="T1 result",
        choices=POS_NEG_NA,
        default=NOT_APPLICABLE,
        help_text="Gaborone and Blantyre only",
        max_length=5,
    )

    crag_t2_result = models.CharField(
        verbose_name="T2 result",
        choices=POS_NEG_NA,
        default=NOT_APPLICABLE,
        help_text="Gaborone and Blantyre only",
        max_length=5,
    )

    class Meta:
        abstract = True



from django import forms
from edc_form_validators import FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin

from .form_validators import LpCsfFormValidator
from .models import LpCsf


class LpCsfForm(SiteModelFormMixin, FormValidatorMixin, forms.ModelForm):
    form_validator_cls = LpCsfFormValidator

    class Meta:
        model = LpCsf
        fields = "__all__"



from edc_lab import RequisitionPanel

from .processing_profiles import (
    csf_chem_processing_profile,
    csf_pkpd_processing_profile,
    csf_stop_processing_profile,
    csf_store_processing_profile,
    qpcr_csf_processing_profile,
)

csf_chemistry_panel = RequisitionPanel(
    name="csf_chem_haem_routine",
    verbose_name="CSF Chem and Haem Routine",
    processing_profile=csf_chem_processing_profile,
)

csf_pkpd_panel = RequisitionPanel(
    name="csf_pk_pd",
    verbose_name="CSF PK/PD",
    processing_profile=csf_pkpd_processing_profile,
)

qpcr_csf_panel = RequisitionPanel(
    name="qpcr_csf",
    verbose_name="qPCR CSF",
    processing_profile=qpcr_csf_processing_profile,
)

csf_panel = RequisitionPanel(
    name="csf_test_and_store",
    verbose_name="CSF Test and Store",
    processing_profile=csf_store_processing_profile,
)

csf_stop_panel = RequisitionPanel(
    name="csf_stop_cm",
    verbose_name="CSF STOP-CM",
    processing_profile=csf_stop_processing_profile,
)



def get_lp_fieldset():
    return (
        "LP",
        {
            "fields": (
                "reason_for_lp",
                "lp_datetime",
                "opening_pressure_measured",
                "opening_pressure",
                "closing_pressure",
                "csf_amount_removed",
            )
        },
    )


def get_csf_fieldset():
    return (
        "CSF",
        {
            "fields": (
                "csf_positive",
                "india_ink",
                "csf_crag_lfa",
                "sq_crag",
                "sq_crag_pos",
                "crf_crag_titre_done",
                "crf_crag_titre",
            )
        },
    )


def get_quantitative_culture_fieldset(requisition_field=None):
    fields = ["qc_assay_datetime", "quantitative_culture"]
    if requisition_field:
        fields.insert(0, requisition_field)
    return (
        "Quantitative Culture",
        {"fields": fields},
    )


def get_csf_culture_fieldset(requisition_field=None):
    fields = [
        "csf_culture_assay_datetime",
        "csf_culture",
        "other_csf_culture",
        "csf_wbc_cell_count",
        "differential_lymphocyte_count",
        "differential_lymphocyte_unit",
        "differential_neutrophil_count",
        "differential_neutrophil_unit",
        "csf_glucose",
        "csf_glucose_units",
        "csf_protein",
        "csf_crag",
        "csf_crag_immy_lfa",
    ]
    if requisition_field:
        fields.insert(0, requisition_field)
    return (
        "CSF culture",
        {"fields": fields},
    )


def get_biosynex_semi_quantitative_crag_fieldset():
    return (
        "Biosynex Semi-quantitative CrAg",
        {
            "fields": [
                "bios_crag",
                "crag_control_result",
                "crag_t1_result",
                "crag_t2_result",
            ]
        },
    )



# Generated by Django 3.2.11 on 2022-02-25 01:19

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_csf", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicallpcsf",
            name="opening_pressure_measured",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=25,
                verbose_name="Was the opening pressure measured",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="lpcsf",
            name="opening_pressure_measured",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=25,
                verbose_name="Was the opening pressure measured",
            ),
            preserve_default=False,
        ),
    ]



from edc_auth.site_auths import site_auths

from .auth_objects import EDC_CSF, EDC_CSF_SUPER, EDC_CSF_VIEW, codenames

site_auths.add_group(*codenames, name=EDC_CSF_VIEW, view_only=True)
site_auths.add_group(*codenames, name=EDC_CSF, no_delete=True)
site_auths.add_group(*codenames, name=EDC_CSF_SUPER)



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_model.validators import datetime_not_future


class QuantitativeCultureModelMixin(models.Model):

    """add a requisition fields if needed, for example:

    qc_requisition = models.ForeignKey(
        get_requisition_model_name(),
        on_delete=PROTECT,
        related_name="qcrequisition",
        verbose_name="QC Requisition",
        null=True,
        blank=True,
        help_text="Start typing the requisition identifier or select one from this visit")
    """

    qc_assay_datetime = models.DateTimeField(
        verbose_name="QC Result Report Date and Time",
        validators=[datetime_not_future],
        blank=True,
        null=True,
    )

    quantitative_culture = models.IntegerField(
        blank=True,
        null=True,
        validators=[MinValueValidator(0), MaxValueValidator(100_000_000)],
        help_text="Units CFU/ml",
    )

    class Meta:
        abstract = True



from django.core.validators import MinValueValidator
from django.db import models
from django.utils.html import format_html
from edc_constants.choices import (
    POS_NEG,
    YES_NO_NA,
    YES_NO_NOT_DONE_AWAITING_RESULTS_NA,
)
from edc_constants.constants import AWAITING_RESULTS, NOT_APPLICABLE
from edc_model.validators import datetime_not_future
from edc_reportable import (
    GRAMS_PER_LITER,
    MILLIGRAMS_PER_DECILITER,
    MILLIMOLES_PER_LITER,
    MM3_DISPLAY,
)

from ..choices import MG_MMOL_UNITS, MM3_PERC_UNITS


class CsfCultureModelMixin(models.Model):

    """add a requisition fields if needed, for example:

    csf_requisition = models.ForeignKey(
        get_requisition_model_name(),
        on_delete=PROTECT,
        related_name="csfrequisition",
        verbose_name="CSF Requisition",
        null=True,
        blank=True,
        help_text="Start typing the requisition identifier or select one from this visit")
    """

    csf_culture_assay_datetime = models.DateTimeField(
        verbose_name="CSF Result Report Date and Time",
        validators=[datetime_not_future],
        null=True,
        blank=True,
    )

    csf_culture = models.CharField(
        verbose_name="CSF Result: Other organism (non-Cryptococcus)",
        max_length=18,
        choices=YES_NO_NOT_DONE_AWAITING_RESULTS_NA,
        default=AWAITING_RESULTS,
        help_text="Complete after getting the results.",
    )

    other_csf_culture = models.CharField(
        verbose_name="If YES, specify organism:", max_length=75, blank=True, null=True
    )

    csf_wbc_cell_count = models.IntegerField(
        verbose_name="Total CSF WBC cell count:",
        help_text=format_html(f"acceptable units are {MM3_DISPLAY}"),
        validators=[MinValueValidator(0)],
        null=True,
        blank=True,
    )

    differential_lymphocyte_count = models.IntegerField(
        verbose_name="Differential lymphocyte cell count:",
        validators=[MinValueValidator(0)],
        blank=True,
        null=True,
        help_text=format_html(f"acceptable units are {MM3_DISPLAY} or %"),
    )

    differential_lymphocyte_unit = models.CharField(
        choices=MM3_PERC_UNITS, max_length=6, null=True, blank=True
    )

    differential_neutrophil_count = models.IntegerField(
        verbose_name="Differential neutrophil cell count:",
        validators=[MinValueValidator(0)],
        blank=True,
        null=True,
        help_text=format_html(f"acceptable units are {MM3_DISPLAY} or %"),
    )

    differential_neutrophil_unit = models.CharField(
        choices=MM3_PERC_UNITS, max_length=6, null=True, blank=True
    )

    csf_glucose = models.DecimalField(
        verbose_name="CSF glucose:",
        decimal_places=1,
        max_digits=3,
        blank=True,
        null=True,
        help_text=f"Units in {MILLIMOLES_PER_LITER} or {MILLIGRAMS_PER_DECILITER}",
    )

    csf_glucose_units = models.CharField(
        verbose_name="CSF glucose units:",
        max_length=6,
        choices=MG_MMOL_UNITS,
        blank=True,
        null=True,
    )

    csf_protein = models.DecimalField(
        verbose_name="CSF protein:",
        decimal_places=2,
        max_digits=4,
        blank=True,
        null=True,
        help_text=f"Units in {GRAMS_PER_LITER}",
    )

    csf_crag = models.CharField(
        verbose_name="CSF CrAg:", max_length=15, choices=POS_NEG, blank=True, null=True
    )

    csf_crag_immy_lfa = models.CharField(
        verbose_name="CSF CrAg done by IMMY CrAg LFA:",
        max_length=5,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    class Meta:
        abstract = True



# Generated by Django 3.2.11 on 2022-02-25 18:42

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_csf", "0004_auto_20220225_0500"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicallpcsf",
            old_name="csf_cr_ag",
            new_name="csf_crag",
        ),
        migrations.RenameField(
            model_name="historicallpcsf",
            old_name="csf_cr_ag_lfa",
            new_name="csf_crag_immy_lfa",
        ),
        migrations.RenameField(
            model_name="historicallpcsf",
            old_name="crag_lfa",
            new_name="csf_crag_lfa",
        ),
        migrations.RenameField(
            model_name="lpcsf",
            old_name="csf_cr_ag",
            new_name="csf_crag",
        ),
        migrations.RenameField(
            model_name="lpcsf",
            old_name="csf_cr_ag_lfa",
            new_name="csf_crag_immy_lfa",
        ),
        migrations.RenameField(
            model_name="lpcsf",
            old_name="crag_lfa",
            new_name="csf_crag_lfa",
        ),
        migrations.RemoveField(
            model_name="historicallpcsf",
            name="csf_results_available",
        ),
        migrations.RemoveField(
            model_name="lpcsf",
            name="csf_results_available",
        ),
        migrations.AlterField(
            model_name="historicallpcsf",
            name="csf_positive",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("PENDING", "Pending")],
                default="PENDING",
                max_length=18,
                verbose_name="CSF positive for cryptococcal meningitis?",
            ),
        ),
        migrations.AlterField(
            model_name="lpcsf",
            name="csf_positive",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("PENDING", "Pending")],
                default="PENDING",
                max_length=18,
                verbose_name="CSF positive for cryptococcal meningitis?",
            ),
        ),
    ]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin

from .admin_site import edc_csf_admin
from .fieldsets import (
    get_biosynex_semi_quantitative_crag_fieldset,
    get_csf_culture_fieldset,
    get_lp_fieldset,
    get_quantitative_culture_fieldset,
)
from .forms import LpCsfForm
from .models import LpCsf


@admin.register(LpCsf, site=edc_csf_admin)
class LpCsfAdmin(ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin):
    form = LpCsfForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_identifier",
                    "report_datetime",
                )
            },
        ),
        get_lp_fieldset(),
        get_csf_culture_fieldset(),
        get_quantitative_culture_fieldset(),
        get_biosynex_semi_quantitative_crag_fieldset(),
        audit_fieldset_tuple,
    )

    radio_fields = {
        "reason_for_lp": admin.VERTICAL,
        "csf_culture": admin.VERTICAL,
        "india_ink": admin.VERTICAL,
        "csf_crag": admin.VERTICAL,
        "csf_crag_lfa": admin.VERTICAL,
        "differential_lymphocyte_unit": admin.VERTICAL,
        "differential_neutrophil_unit": admin.VERTICAL,
        "csf_glucose_units": admin.VERTICAL,
        "bios_crag": admin.VERTICAL,
        "crag_control_result": admin.VERTICAL,
        "crag_t1_result": admin.VERTICAL,
        "crag_t2_result": admin.VERTICAL,
    }

    list_display = ("lp_datetime", "reason_for_lp")

    list_filter = ("lp_datetime", "reason_for_lp")



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils.html import format_html
from edc_constants.choices import YES_NO
from edc_model.validators import datetime_not_future

from ..choices import LP_REASON


class LpModelMixin(models.Model):
    lp_datetime = models.DateTimeField(
        verbose_name="LP Date and Time", validators=[datetime_not_future]
    )

    reason_for_lp = models.CharField(
        verbose_name="Reason for LP", max_length=50, choices=LP_REASON
    )

    opening_pressure_measured = models.CharField(
        verbose_name="Was the opening pressure measured", max_length=25, choices=YES_NO
    )

    opening_pressure = models.IntegerField(
        validators=[MinValueValidator(1), MaxValueValidator(99)],
        blank=True,
        null=True,
        help_text=format_html("Units cm of H<sub>2</sub>O"),
    )

    closing_pressure = models.IntegerField(
        blank=True,
        null=True,
        validators=[MinValueValidator(0), MaxValueValidator(99)],
        help_text=format_html("Units cm of H<sub>2</sub>O"),
    )

    csf_amount_removed = models.IntegerField(
        blank=True,
        null=True,
        verbose_name="CSF amount removed",
        validators=[MinValueValidator(1)],
        help_text="Units ml",
    )

    class Meta:
        abstract = True



from edc_lab import AliquotType

qfc = AliquotType(name="Quantitative FC", alpha_code="QFC", numeric_code="61")

csf_store = AliquotType(name="CSF store", alpha_code="CSF", numeric_code="62")

csf_testing = AliquotType(name="Isolates", alpha_code="ISOLATES", numeric_code="64")

csf_glucose = AliquotType(name="Glucose", alpha_code="GLUCOSE", numeric_code="65")

csf_protein = AliquotType(name="Protein", alpha_code="PROTEIN", numeric_code="66")

csf_pellet = AliquotType(name="CSF Pellet", alpha_code="PELLET", numeric_code="67")

csf_supernatant = AliquotType(
    name="CSF Supernatant", alpha_code="SUPERNATANT", numeric_code="68"
)

csf = AliquotType(name="Cerebro Spinal Fluid", alpha_code="CSF", numeric_code="56")
csf.add_derivatives(
    qfc,
    csf_store,
    csf_testing,
    csf_glucose,
    csf_protein,
    csf,
    csf_pellet,
    csf_supernatant,
)



# Generated by Django 3.2.11 on 2022-02-25 01:23

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_csf", "0002_auto_20220225_0419"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicallpcsf",
            name="opening_pressure",
            field=models.IntegerField(
                blank=True,
                help_text="Units cm of H<sub>2</sub>O",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(99),
                ],
            ),
        ),
        migrations.AlterField(
            model_name="lpcsf",
            name="opening_pressure",
            field=models.IntegerField(
                blank=True,
                help_text="Units cm of H<sub>2</sub>O",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(99),
                ],
            ),
        ),
    ]



# Generated by Django 3.2.11 on 2022-02-28 17:32

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_csf", "0005_auto_20220225_2142"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicallpcsf",
            name="csf_culture",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("awaiting_results", "Awaiting results"),
                    ("not_done", "Not done"),
                    ("N/A", "Not applicable"),
                ],
                default="awaiting_results",
                help_text="Complete after getting the results.",
                max_length=18,
                verbose_name="CSF Result: Other organism (non-Cryptococcus)",
            ),
        ),
        migrations.AlterField(
            model_name="historicallpcsf",
            name="csf_positive",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("awaiting_results", "Awaiting results"),
                ],
                default="awaiting_results",
                max_length=18,
                verbose_name="CSF positive for cryptococcal meningitis?",
            ),
        ),
        migrations.AlterField(
            model_name="lpcsf",
            name="csf_culture",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("awaiting_results", "Awaiting results"),
                    ("not_done", "Not done"),
                    ("N/A", "Not applicable"),
                ],
                default="awaiting_results",
                help_text="Complete after getting the results.",
                max_length=18,
                verbose_name="CSF Result: Other organism (non-Cryptococcus)",
            ),
        ),
        migrations.AlterField(
            model_name="lpcsf",
            name="csf_positive",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("awaiting_results", "Awaiting results"),
                ],
                default="awaiting_results",
                max_length=18,
                verbose_name="CSF positive for cryptococcal meningitis?",
            ),
        ),
    ]
